
/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
void DrawPixel(SDL_Surface *screen, int x, int y, Uint8 Red, Uint8 Green, Uint8 Blue) 
{ 
    Uint32 color = SDL_MapRGB(screen->format, Red, Green, Blue);
    switch (screen->format->BytesPerPixel) 
    { 
	case 1: // 8-bpp 
	{ 
	    Uint8 *bufp; 
	    bufp = (Uint8 *)screen->pixels + y*screen->pitch + x; 
	    *bufp = color; 
	} 
	    break; 
	case 2: // 15-bpp or 16-bpp 
	{ 
	    Uint16 *bufp; 
	    bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x; 
	    *bufp = color; 
	} 
	    break; 
	case 3: // 24-bpp mode, usually not used 
	{ 
	    Uint8 *bufp; 
	    bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3; 
	    if(SDL_BYTEORDER == SDL_LIL_ENDIAN) 
	    { 
		bufp[0] = color; 
		bufp[1] = color >> 8; 
		bufp[2] = color >> 16; 
	    } else { 
		bufp[2] = color; 
		bufp[1] = color >> 8; 
		bufp[0] = color >> 16; 
	    } 
	} 
	    break; 
	case 4: // 32-bpp 
	{ 
	    Uint32 *bufp; 
	    bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x; 
	    *bufp = color; 
	} 
	    break; 
    } 
}

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
void DisplayOnScreen(SDL_Surface *imageO, SDL_Surface * screen, int width, int height, unsigned char *VidY, double *VidU2, double *VidV2, int ms_frame_lap)
{

 double tmpr, tmpg, tmpb;
 int i,j,b;

	if( SDL_MUSTLOCK(imageO) )
		b = SDL_LockSurface(imageO);

	for (i=0;i<height;i++)	{
	    for (j=0;j<width;j++)	{
			tmpr = 1.0016 * VidY[i*width+j] + 0.0027706 * VidU2[i*width+j] + 1.5748 * VidV2[i*width+j];
			tmpg = 0.99953 * VidY[i*width+j] - 0.18815 * VidU2[i*width+j] - 0.46812 * VidV2[i*width+j];
			tmpb = 1.0000 * VidY[i*width+j] + 1.8556 * VidU2[i*width+j] + 0.00010574 * VidV2[i*width+j];
			if (tmpr < 0)	tmpr = 0;
			if (tmpg < 0)	tmpg = 0;
			if (tmpb < 0)	tmpb = 0;
			if (tmpr > 255)	tmpr = 255;
			if (tmpg > 255)	tmpg = 255;
			if (tmpb > 255)	tmpb = 255;
			DrawPixel(imageO, j, i,  (Uint8) (tmpr), (Uint8) (tmpg), (Uint8) (tmpb));
	    }
	}

	if( SDL_MUSTLOCK(imageO) )
    	SDL_UnlockSurface(imageO);

	SDL_Rect rcDest = { 0, 0, width, height };
	SDL_BlitSurface(imageO,0,screen,&rcDest);
	SDL_Flip(screen);

	// 40 ms between frames (+- 10 if we press UP or DOWN)
	msleep(ms_frame_lap);	
}

/*----------------------------------------------------*/
/*		      Open Double data file                   */
/*----------------------------------------------------*/
double * OpenDoubleFile(int SizeRaws, int SizeColumns, char *name)
{
int i,j;
FILE *NameFile;
double *Data;
double tmp;

    Data = (double *) calloc (SizeRaws*SizeColumns, sizeof(double));

    if ((NameFile = fopen(name, "r")) == NULL)	{
        printf("\n\nProblem !! Can't find the file named '%s'\n\n",name);
        exit(0);	}

    for (i=0;i<SizeRaws;i++)
    {
        for (j=0;j<SizeColumns;j++)
        {
            fscanf (NameFile, "%lf ", &tmp);
            Data[i*SizeColumns+j] = tmp;
        }
        fscanf(NameFile,"\n");
    }	
    fclose(NameFile);

    return(Data);
}

/*----------------------------------------------------*/
/*		Store Double data into file                   */
/*----------------------------------------------------*/
void StoreDoubleFile(int SizeRaws, int SizeColumns, char name[256], double * Data)
{
	int i,j;
	FILE *NameFile;
	
	//    sprintf(name,"./Images/%s",name);
    NameFile = fopen(name, "w");
    for (i=0;i<SizeRaws;i++)	{
        for (j=0;j<SizeColumns;j++)
            fprintf (NameFile, "%8.8lf ", Data[i*SizeColumns+j]);
		//            fprintf (NameFile, "%lf ", Data[i*SizeColumns+j]);
        fprintf(NameFile,"\n");
    }	
	
    fclose(NameFile);
}

/*----------------------------------------------------*/
/*		    Store UC data into file                   */
/*----------------------------------------------------*/
void StoreUCFile(int SizeRaws, int SizeColumns, char name[256], unsigned char * Data)
{
	int i,j;
	FILE *NameFile;
	
	//    sprintf(name,"./Images/%s",name);
    NameFile = fopen(name, "w");
    for (i=0;i<SizeRaws;i++)	{
        for (j=0;j<SizeColumns;j++)
            fprintf (NameFile, "%d ", Data[i*SizeColumns+j]);
		//            fprintf (NameFile, "%lf ", Data[i*SizeColumns+j]);
        fprintf(NameFile,"\n");
    }	
	
    fclose(NameFile);
}

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
// Read the PPM header 
void ReadHeader(FILE *FileName, int *Cols, int *Rows)
{
  char ch;
  int  maxval;

  if (fscanf(FileName, "P%c\n", &ch) != 1 || ch != '6') 
    { printf("file is not in ppm raw format; cannot read"); exit(0); }

  // skip comments
  ch = getc(FileName);
  while (ch == '#')    {
      do {
	ch = getc(FileName);
      } while (ch != '\n');	// read to the end of the line
      ch = getc(FileName);
    }

  ungetc(ch, FileName);		// put that digit back

  // read the width, height, and maximum value for a pixel 
  fscanf(FileName, "%d%d%d\n", Cols, Rows, &maxval);

  if (maxval != 255) { printf("image is not true-color (24 bit); read failed"); exit(0); }
  
}

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
// Read the PPM image :
unsigned char * ReadPPM(char *filename)
{
  int Cols, Rows, num, size;
  unsigned char *MyImage; 

  unsigned char *image = (unsigned char *) malloc(sizeof(unsigned char));
  FILE  *FileName    = fopen(filename, "r");

  if (!image) { printf("cannot allocate memory for new image"); exit(0); }
  if (!FileName)    { printf("cannot open file for reading"); exit(0); }

  ReadHeader(FileName, &Cols, &Rows);

  size          = Cols * Rows * 3;
  
  MyImage = (unsigned char *) malloc(3*(Cols)*(Rows));

  if (!image) { printf("cannot allocate memory for new image"); exit(0); }

  num = fread((MyImage), 1, (size_t) size, FileName);

  if (num != size) { printf("cannot read image data from file"); exit(0); }

  fclose(FileName);

	return (MyImage);
}

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
// Write the PPM image :
void WritePPM(unsigned char *image, int Rows, int Cols, char *filename)
{
  int num;
  int size = Rows * Cols * 3;

  FILE *FileName = fopen(filename, "w");

  if (!FileName) { printf("cannot open file for writing"); exit(0); }

  fprintf(FileName, "P6\n%d %d\n%d\n", Cols, Rows, 255);

  num = fwrite(image, 1, (size_t) size, FileName);

  if (num != size) { printf("cannot write image data to file"); exit(0); }

  fclose(FileName);
}  

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
// Read the PGM header 
void ReadPGMHeader(FILE *FileName, int *Cols, int *Rows)
{
	char ch;
	int  maxval;
	
	if (fscanf(FileName, "P%c\n", &ch) != 1 || ch != '5') 
    { printf("file is not in pgm raw format; cannot read\n\n"); exit(0); }
	
	// skip comments
	ch = getc(FileName);
	while (ch == '#')    {
		do {
			ch = getc(FileName);
		} while (ch != '\n');	// read to the end of the line
		ch = getc(FileName);
    }
	
	ungetc(ch, FileName);		// put that digit back
	
	// read the width, height, and maximum value for a pixel 
	fscanf(FileName, "%d%d%d\n", Cols, Rows, &maxval);
	
	if (maxval != 255) { printf("image is not true-color (24 bit); read failed\n\n"); exit(0); }
	
}

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
// Read the PGM image :
unsigned char * ReadPGM(char *filename, int *PGM_Rows, int *PGM_Cols)
{
	int Cols, Rows;
	int num, size;
	unsigned char *MyImage; 
	
	unsigned char *image = (unsigned char *) malloc(sizeof(unsigned char));
	FILE  *FileName    = fopen(filename, "r");
	
	if (!image) { printf("cannot allocate memory for new image\n\n"); exit(0); }
	if (!FileName)    { printf("cannot open file for reading\n\n"); exit(0); }
	
	ReadPGMHeader(FileName, &Cols, &Rows);
	
	size          = Cols * Rows;
	
	*PGM_Rows = Rows;
	*PGM_Cols = Cols;
	
	MyImage = (unsigned char *) malloc((Cols)*(Rows));
	
	if (!image) { printf("cannot allocate memory for new image\n\n"); exit(0); }
	
	num = fread((MyImage), 1, (size_t) size, FileName);
//	printf("%d %d\n",num,size);
	if (num != size) { printf("cannot read image data from file\n\n");  printf("%d \t %d\n",size,num); exit(0); }
	
	fclose(FileName);
	
	return (MyImage);
}

/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
// Write the PGM image :
void WritePGM(unsigned char *image, int Rows, int Cols, char *filename)
{
	int num;
	int size = Rows * Cols;
	
	FILE *FileName = fopen(filename, "w");
	
	if (!FileName) { printf("cannot open file for writing\n\n"); exit(0); }
	
	fprintf(FileName, "P5\n%d %d\n%d\n", Cols, Rows, 255);
	
	num = fwrite(image, 1, (size_t) size, FileName);
	
	if (num != size) { printf("cannot write image data to file\n\n"); exit(0); }
	
	fclose(FileName);
}  

/*--------------------------------------------------------------------------*/
void readline(FILE *in_file, char *string)
{
int c;
char slashes[4];
int skip;
int d;

	strcpy(slashes,"//");

	do {
		skip = FALSE;
		if ( (c = fscanf(in_file, "%[^\n]\n", string) != EOF )) {
				d = strncmp(string, slashes, 2);
			if ( d == 0 ) skip = TRUE;
		}
		else {
			printf("invalid parameters file\n");
			exit(0);
		}
	}
	while(skip);
}


/*--------------------------------------------------------------------------*/
/*
void ReadSettings(char *name) {
char line[100];
FILE *in;

	//Opens the parameters file and read content
	in = fopen(name,"r");

	readline(in, line);	sscanf(line, "%s", FileName);
	readline(in, line);	sscanf(line, "%d", &height);
	readline(in, line);	sscanf(line, "%d", &width);

	fclose(in);
}
*/
